@@ -1,6 +1,6 @@
 /* Routines for handling XML generic OS data provided by target.
 
-   Copyright (C) 2008-2012 Free Software Foundation, Inc.
+   Copyright (C) 2008-2014 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -22,7 +22,7 @@
 #include "vec.h"
 #include "xml-support.h"
 #include "osdata.h"
-#include "gdb_string.h"
+#include <string.h>
 #include "ui-out.h"
 #include "gdbcmd.h"
 
