@@ -1,6 +1,5 @@
 /* Remote target callback routines.
-   Copyright 1995-1997, 2000, 2002-2004, 2007-2012 Free Software
-   Foundation, Inc.
+   Copyright 1995-2014 Free Software Foundation, Inc.
    Contributed by Cygnus Solutions.
 
    This file is part of GDB.
@@ -24,6 +23,7 @@
 #ifdef HAVE_CONFIG_H
 #include "cconfig.h"
 #endif
+#include "config.h"
 #include "ansidecl.h"
 #include <stdarg.h>
 #include <stdio.h>
